#include <iostream>
#include <cmath>
#include <fstream>
#include <cstdlib>
#include <ctime>

using namespace std;

double potential(double x)
{
  return x*x*x*x - 3.0*x*x + x/2.0;
}
double urand1()
{
  return (rand()*1.0)/RAND_MAX;
}
double urand2()
{
  double random = (rand()*1.0)/RAND_MAX;
  for(int i = 1; i <= 30; i++)
  {
    if(random < (1.0/30.0)*i)
      return -2.0 + ((4.0/30.0)*i);
  }
}
void double_well(double T)
{
  double range = 4.0;
  double number[30];
  int visited[30];
  double xa = 0.0;
  double xb;
  double prob;
  double random_compare;
  double energy;
  double step = range/30.0;
  number[0] = -2.0;
  visited[0] = 0.0;
  for(int i = 1; i <= 29; i++)
  {
    number[i] = number[i-1] + step;
    visited[i] = 0.0;
  }
  for(int N = 0; N <= pow(10,6); N++)
  {
    xb = urand2();
    prob = exp(-(potential(xb)-potential(xa))/T);
    random_compare = urand1();
    if(prob > random_compare)
    {
      xa = xb;  
    }
    for(int j = 0; j <= 29; j++)
    {
      if(xa <= number[j+1] && xa >= number[j])
	visited[j]++;
    }
  } 
  double sum_states = 1000000.0;
  double interval_prob[30];
  for(int l = 0; l <= 29; l++)
  {
    interval_prob[l] = (visited[l]/sum_states);
    cout << interval_prob[l] << ' ' << number[l] << endl;
  }

}
void avg_energy()
{
  ofstream outfile("avg_energy.txt");
  ofstream outfile2("cv.txt");
  double range = 4.0;
  double number[30];
  int visited[30];
  double PE[30];
  double PE2[30];
  double xa = 0.0;
  double xb;
  double prob;
  double random_compare;
  double energy;
  double sum_states;
  double cv;
  double cv_1;
  double sum_E;
  double step = range/30.0;
  number[0] = -2.0;
  visited[0] = 0.0;
  PE[0] = potential(-2.0);
  for(int i = 1; i <= 29; i++)
  {
    number[i] = number[i-1] + step;
    visited[i] = 0.0;
    PE[i] = potential(-2.0+(step*i));
    PE2[i] = potential(pow(-2.0+(step*i),2));
  }
  for(double T = 0.1; T <= 2.0; T += (1.9/50))
  {
    for(int i = 1; i <= 29; i++)
    {
      number[i] = number[i-1] + step;
      visited[i] = 0.0;
      PE[i] = potential(-2.0+(step*i));
      PE2[i] = potential(pow(-2.0+(step*i),2));
    }
    for(int N = 0; N <= pow(10,6); N++)
    {
      xb = urand2();
      prob = exp(-(potential(xb)-potential(xa))/T);
      random_compare = urand1();
      if(prob > random_compare)
      {
	xa = xb;  
      }
      for(int j = 0; j <= 29; j++)
      {
	if(xa <= number[j+1] && xa >= number[j])
	  visited[j]++;
      }
    } 
    sum_states = 0.0;
    sum_E = 0.0;
    cv_1 = 0.0;
    for(int l = 0; l <= 29; l++)
    {
      sum_states += visited[l]/2.0;
    }

    for(int k = 0; k <= 29; k++)
    {
      sum_E += (visited[k]/2.0)*PE[k];
      cv_1 += (visited[k]/2.0)*pow(PE[k],2);
    } 
    sum_E /= (sum_states);
    cv_1 /= (sum_states);
    cv = (1/pow(T,2))*(cv_1 - pow(sum_E,2));
    outfile << sum_E << ' ' << T << endl;
    outfile2 << cv << ' ' << T << endl;
  }
  
}
int main(int argc, char*argv[])
{
  srand(time(0));
  double T = atof(argv[1]);
  if(atoi(argv[2]) == 1)
    double_well(T);
  if(atoi(argv[2]) == 2)
    avg_energy();
  return 0;
}
